class Measurement:
    def __init__(self, val, unc):
        self.val = val
        self.unc = unc
    def __add__(self, m2):
        val = self.val + m2.val
        unc = norm(self.unc, m2.unc)
        return Measurement(val, unc)
    def __sub__(self, m2):
        val = self.val - m2.val
        unc = norm(self.unc, m2.unc)
        return Measurement(val, unc)
    def __mul__(self, m2):
        if isinstance(m2, Measurement):
            val = self.val*m2.val
            unc = val*norm(self.unc/self.val, m2.unc/m2.val)
            return Measurement(val, unc)
        #Otherwise, it's a scalar
        else:
            return Measurement(self.val*m2, self.unc*m2)
    def __truediv__(self, m2):
        if isinstance(m2, Measurement):
            val = self.val/m2.val
            unc = val*norm(self.unc/self.val, m2.unc/m2.val)
            return Measurement(val, unc)
        else:
            return Measurement(self.val/m2, self.unc/m2)
    def __pow__(self, exp):
        return Measurement(self.val**exp, exp*(self.val)**(exp-1)*self.unc)
    def __str__(self):
        #TODO: improve this printing
        #TODO: Figure out how to use localization(E.g. US vs European notation, etc)
        return f"{self.val:.3f} \\pm {self.unc:.3f}"

def norm(*x):
    return (sum([i**2 for i in x]))**0.5


from .measurement import Measurement

def read(filename, headerline = False, col = 0, totalcol = 1):
    #Reads the data from the file.
    #The file MUST have only the raw data
    with open(filename, 'r') as f:
        s = f.read()
        if headerline:
            s = s.split('\n', 1)[1]
        #For European/Brazilian number notation
        s = s.replace(",", ".")
        data = [float(val) for i, val in enumerate(s.split()) if i%totalcol == col]
    return data

#The standard deviation of the mean
def sigma_m(data):
    l = len(data)
    av = sum(data)/l
    return ( sum([(x-av)**2 for x in data])/((l-1)*l) ) ** 0.5

def create(filename):
    data = read(filename)
    return Measurement( sum(data)/len(data), sigma_m(data) )


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="u-calc",
    version="0.0.1",
    author="vini-fda, pekpuglia",
    author_email="",
    description="Calculator with uncertainties",
    license="GPLv3",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/vini-fda/u-calc",
    packages=setuptools.find_packages(),
    classifiers=[
        "Topic :: Scientific/Engineering :: Physics",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)